@@ -32,38 +32,34 @@ def match(x0, x, match_type=1):
         return np.searchsorted(x, x0, side="right") - 1
 
 
-def p_max(params):
+def clean_params_for_xlsx_fns(params):
     p_all = []
     for param in params:
         if hasattr(param, "__len__"):
-            if param.size == 1:
+            if hasattr(param, 'size') and param.size == 1:
                 param = [np.asscalar(param)]
-            p_all += list(param)
-        else:
+            pnew = [p for p in param if not isinstance(p, str)]
+            p_all += pnew
+        elif not isinstance(param, str):
             p_all += [param]
-    return max(p_all)
+    if not len(p_all):
+        return None
+    return p_all
+
+
+def p_max(params):
+    p_all = clean_params_for_xlsx_fns(params)
+    return None if p_all is None else max(p_all)
 
 
 def p_min(params):
-    p_all = []
-    for param in params:
-        if hasattr(param, "__len__"):
-            if param.size == 1:
-                param = [np.asscalar(param)]
-            p_all += list(param)
-        else:
-            p_all += [param]
-    return min(p_all)
+    p_all = clean_params_for_xlsx_fns(params)
+    return None if p_all is None else min(p_all)
 
 
 def p_sum(params):
-    p_all = 0
-    for param in params:
-        if hasattr(param, "__len__"):
-            p_all += sum(param)
-        else:
-            p_all += param
-    return p_all
+    p_all = clean_params_for_xlsx_fns(params)
+    return None if p_all is None else sum(p_all)
 
 
 def lookup(x, x0, y, approx=True):
@@ -80,7 +76,7 @@ def lookup(x, x0, y, approx=True):
     if isinstance(x[0], str):
         x0 = str(x0)
     if not approx:  # need exact match
-        return y[np.where(x0 == x)[0][0]]
+        return y[np.where(x0 == np.array(x))[0][0]]
     else:
         inds = np.searchsorted(x, x0, side='right') - 1
         return y[inds]
@@ -100,7 +96,7 @@ def vlookup(x0, vals, ind, approx=True):
     if isinstance(vals[0][0], str):
         x0 = str(x0)
     if not approx:  # need exact match
-        return vals[int(ind)][np.where(x0 == vals[0])[0][0]]
+        return vals[int(ind)][np.where(x0 == np.array(vals[0]))[0][0]]
     else:
         inds = np.searchsorted(vals[0], x0, side='right') - 1
         return vals[ind][int(inds)]
@@ -113,19 +109,31 @@ def vlookup(x0, vals, ind, approx=True):
 #     return y[inds]
 
 
-def p_or(**args):
+def p_or(*args):
     if len(args) != 2:
         return np.logical_or.reduce(np.array(args))
     return np.logical_or(*args)
 
 
-def p_and(**args):
+def p_and(*args):
     if len(args) != 2:
         return np.logical_and.reduce(np.array(args))
     return np.logical_and(*args)
 
 
 def concat(parts):
+    if isinstance(parts[0], (list, np.ndarray)):
+        max_len = len(parts[0])
+        for i, part in enumerate(parts):
+            if not isinstance(parts[i], (list, np.ndarray)):
+                parts[i] = [part] * max_len
+        new_list = []
+        for i in range(len(parts[0])):
+            sub_list = []
+            for j in range(len(parts)):
+                sub_list.append(str(parts[j][i]))
+            new_list.append(''.join(sub_list))
+        return new_list
     return ''.join([str(x) for x in parts])
 
 
